//******************************************************************************
// RO_PINOSC_TA0_WDTp_One_Button example
// Touch the middle element to turn on/off the center button LED
// RO method capactiance measurement using PinOsc IO, TimerA0, and WDT+
//******************************************************************************
#include "CTS_Layer.h"
#include "i2c_functions.h"
#include "lcd_aqm0802.h"
#include "ina226.h"

void delayss(volatile unsigned int ss);
void sleep(unsigned int time);
void usi_send1(unsigned int data);
void disp_watt(long data);
void disp_volt(long data);
void disp_curr(long data);

// Uncomment to have this compiler directive run characterization functions only
// Comment to have this compiler directive run example application
//#define ELEMENT_CHARACTERIZATION_MODE	

#define DEBUG 1			// SPI out is 1 set
#define DELAY 5000 		// Timer delay timeout count - 5000*0.1msec = 500 msec
#define LEDoFF		(P2OUT &= ~BIT7)
#define LEDoN		(P2OUT |= BIT7)
#define SLEEP_SHORT 2000	// 200ms
#define SLEEP_NORMAL 3500	// 350ms
#define SLEEP_LONG 5000		// 500ms
#define BUFMAX 10

#ifdef ELEMENT_CHARACTERIZATION_MODE
// Delta Counts returned from the API function for the sensor during characterization
unsigned int dCnt;
#endif


unsigned int debug_meascount;
long calc_curr=0,calc_volt=0;	// vZl
int disp_mode=0;				// \[h؂ւ
// Main Function
void main(void){
	uint16_t  sleep_interval = SLEEP_NORMAL;// Timer A ݒl

	// Initialize System Clocks
	WDTCTL = WDTPW + WDTHOLD;			// Stop watchdog timer
	BCSCTL1 = CALBC1_1MHZ;				// Set DCO to 1, 8, 12 or 16MHz
	DCOCTL = CALDCO_1MHZ;
	BCSCTL2 |= DIVS_2;					// divide SMCLK by 4 for 250khz
	BCSCTL3 |= LFXT1S_2;				// LFXT1 = VLO

	P1OUT = 0x00;						// Clear Port 1 bits
	P1DIR |= BIT0+BIT1+BIT2+BIT3+BIT4+BIT5;	// Set P1.0 as output pin

	P2SEL &= ~(BIT6 + BIT7);				// Configure XIN (P2.6) and XOUT (P2.7) to GPIO
	P2OUT = 0x00;							// Drive all Port 2 pins low
	P2DIR = 0xFF;							// Configure all Port 2 pins outputs

/** USI I2C ݒ */
	P1SEL = 0x60;					// 6,5 USI mode
	P1DIR = 0xFF;					// Set P1 all output direction

	USICTL0 |= USISWRST;
	USICTL0 &= ~USISWRST;						// start USI
	USICTL0  |= USIPE7 + USIPE6 + USIMST;
	USICTL1  = USII2C;							// I2C mode
	USICKCTL = USIDIV_2 + USISSEL_2 + USICKPL;	// div = 128, SMCLK,

	LEDoN;

	// Initialize Baseline measurement
	TI_CAPT_Init_Baseline(&one_button);

	// Update baseline measurement (Average 5 measurements)
	TI_CAPT_Update_Baseline(&one_button,5);

	i2c_stop();
	ina226_init();				// ResetAVGݒ
	sleep(100);

	lcd_init_1st();
	sleep(2300);				//Timer A0(230ms @ 12KHz)
	lcd_init_2nd();
	delayss(2000);				// 1ms maybe
	LEDoFF;

	lcd_locate(0x00);
	lcd_puts("USB ");
	i2c_start(LCD_ADDR,I2C_RW_WRITE);
	i2c_send(0x40);		// Data set
	i2c_send(0xC0);		//@^
	i2c_send(0xAF);		//@b
	i2c_send(0xC1);		//@`
	i2c_stop();

	lcd_locate(0x40);
	i2c_start(LCD_ADDR,I2C_RW_WRITE);
	i2c_send(0x40);		// Data set
	i2c_send(0xD2);		//@
	i2c_send(0xB0);		//@[
	i2c_send(0xC0);		//@^
	lcd_puts2("(226)");
	i2c_stop();
	sleep(10000);		// 1000ms

 // Main loop starts here
  while (1)  {
  	#ifdef ELEMENT_CHARACTERIZATION_MODE
	// Get the raw delta counts for element characterization 
	TI_CAPT_Custom(&one_button,&dCnt);
	__no_operation(); 					// Set breakpoint here	
	#endif
	  	
	#ifndef ELEMENT_CHARACTERIZATION_MODE	  	
	// Check if the middle element sensor has been triggered. The API call
	// compares the value from the sensor against the threshold to determine
	// trigger condition  	
	if(TI_CAPT_Button(&one_button)){
		disp_mode++;
		if(disp_mode>1)disp_mode=0;
	}
//	LEDoN;

	switch( disp_mode ){
	case 0:
		calc_volt = (long)ina226_voltage() / 10;	// mVPʂȂ̂ŁA_ʒu
		calc_curr = (long)ina226_current();
		lcd_locate(0x00);		// 1s0P^
		disp_volt(calc_volt);
		lcd_locate(0x40);		// 2s0P^
		disp_curr(calc_curr);
	break;
	case 1:
#if 0
		calc_volt = (long)ina226_power();			// mWP 25mW/bit
		calc_curr = (long)ina226_current();
#else
		calc_volt = (long)ina226_voltage();			// mVP
		calc_curr = (long)ina226_current();
		calc_volt = calc_volt * calc_curr / 1000;	// mWP
#endif
		lcd_locate(0x00);		// 1s0P^
		disp_watt(calc_volt);
		lcd_locate(0x40);		// 2s0P^
		disp_curr(calc_curr);
	break;
	default:
	break;
	}

//	LEDoFF;
	if(calc_curr<100){
		sleep_interval=SLEEP_LONG;
	} else {
		sleep_interval=SLEEP_NORMAL;
	}
	if(calc_curr>300)sleep_interval=SLEEP_SHORT;

// Put the MSP430 into LPM3 for a certain DELAY period
    sleep(sleep_interval);
    #endif

  }
} // End Main

//#if 0
// USI 荞݃[`
#pragma vector=USI_VECTOR
__interrupt void universal_serial_interface(void)
{
	USICTL1 &= ~USIIFG;
}
//#endif
/******************************************************************************/
// Timer0_A0 Interrupt Service Routine: Disables the timer and exists LPM3  
/******************************************************************************/
#pragma vector=TIMER0_A0_VECTOR
__interrupt void ISR_Timer0_A0(void)
{
  TA0CTL &= ~(MC_1);
  TA0CCTL0 &= ~(CCIE);
  __bic_SR_register_on_exit(LPM3_bits+GIE);
}

#pragma vector=PORT2_VECTOR,             \
  PORT1_VECTOR,                          \
  TIMER0_A1_VECTOR,                      \
  NMI_VECTOR,COMPARATORA_VECTOR,         \
  ADC10_VECTOR
__interrupt void ISR_trap(void)
{
  // the following will cause an access violation which results in a PUC reset
  WDTCTL = 0;
}

/* ----------- l͓K
 * @16MHz :: ss=40000 is about 40ms  ss=4000  is about 4ms */
void delayss(volatile unsigned int ss){
	ss = ss / 16 ;	// DCO=1MHz
    for( ; ss>0 ; ss--){
		;
	}
}
void delayss_r(char rep){
	if(rep>0){
		while(rep--){
			delayss(60000);
		}
	}
}

// Sleep Function
// Configures Timer A to run off ACLK, count in UP mode, places the CPU in LPM3
// and enables the interrupt vector to jump to ISR upon timeout
void sleep(unsigned int time){
    TA0CCR0 = time;
    TA0CTL = TASSEL_1+MC_1+TACLR;
    TA0CCTL0 &= ~CCIFG;
    TA0CCTL0 |= CCIE;
    __bis_SR_register(LPM3_bits+GIE);
    __no_operation();
}
void usi_send1(unsigned int data){
	char ddram=0x00;
	i2c_start(LCD_ADDR,I2C_RW_WRITE);
	i2c_send(0x80);				// Command set
	i2c_send(0x80 | ddram);		// set DDRAM AhX bit7AhXݒrbg
	delayss(50);				// about 40us Ԃ
	i2c_send(0x40);				// Data set
	i2c_send( ((data/10000)%10)+0x30 );
	i2c_send( ((data/1000 )%10)+0x30 );
	i2c_send( ((data/100  )%10)+0x30 );
	i2c_send( ((data/10   )%10)+0x30 );
	i2c_send( ((data/1    )%10)+0x30 );
	i2c_stop();
}
void disp_watt(long data){
	i2c_start(LCD_ADDR,I2C_RW_WRITE);
	i2c_send(0x40);				// Data set
	lcd_puts2("P:");
	i2c_send( ((data/1000)%10)+0x30 );
	lcd_puts2(".");
	i2c_send( ((data/100 )%10)+0x30 );
	i2c_send( ((data/10  )%10)+0x30 );
	i2c_send( ((data/1   )%10)+0x30 );
	lcd_puts2("W ");
	i2c_stop();
}
void disp_volt(long data){
#if 1
	i2c_start(LCD_ADDR,I2C_RW_WRITE);
	i2c_send(0x40);				// Data set
	lcd_puts2("V:");
	i2c_send( ((data/100)%10)+0x30 );
	lcd_puts2(".");
	i2c_send( ((data/10 )%10)+0x30 );
	i2c_send( ((data/1  )%10)+0x30 );
	lcd_puts2("V ");
	i2c_stop();
#else
	char sss[20];	// i[p
	int aa1,aa2,aa3;
	aa1 = (int)(data/100)%10;
	aa2 = (int)(data/10 )%10;
	aa3 = (int)(data/1  )%10;
	sprintf( sss , "v:%d.%d%dV ",aa1,aa2,aa3 );
	lcd_puts( sss );
#endif
}

void disp_curr(long data){
	i2c_start(LCD_ADDR,I2C_RW_WRITE);
	i2c_send(0x40);				// Data set
	lcd_puts2("C:");
	i2c_send( ((data/1000)%10)+0x30 );
	lcd_puts2(".");
	i2c_send( ((data/100 )%10)+0x30 );
	i2c_send( ((data/10  )%10)+0x30 );
	i2c_send( ((data/1   )%10)+0x30 );
	lcd_puts2("A");
	i2c_stop();
}

